import sys
import datetime


def getFileLocation(arg):
    try:
        return sys.argv[arg]
    except IndexError:  # if no file location is given
        print('No file location given.  Example: /home/user/yato.json')
        sys.exit()


def getTask(arg):
    try:
        return sys.argv[arg]
    except IndexError:  # if no task is given
        print('No task given.')
        sys.exit()


def getNewName(arg):
    try:
        return sys.argv[arg]
    except IndexError:  # if no new name is given
        print('No new name given.')
        sys.exit()


def getCmd():
    try:
        return sys.argv[1]
    except IndexError:
        print('No command given, use -h or --help for help.')
        sys.exit()


def validDate(date):
    try:
        datetime.datetime(int(date[2]), int(date[1]), int(date[0]))
        return True
    except ValueError:
        return False


def getDate(arg):
    try:
        date = sys.argv[arg].split('/')
        if validDate(date):
            return date
        else:
            print('Invalid date.  Example: 01/01/2020')
            sys.exit()
    except IndexError:  # if no date is given
        print('No date given.')
        sys.exit()


def getuInt(arg):
    try:
        integer = int(sys.argv[arg])
        if integer > 0:
            return integer
        else:
            print('Invalid integer, must be positive.')
            sys.exit()
    except ValueError:
        print('Invalid, please enter a number.')
        sys.exit()



import pytest
import json
import yato.actions as actions
from JSONManip import getFile as getFile
import os
from pathlib import Path
from shutil import rmtree


def remove():
    removeFiles = [
        'config.ini',
        'lists.json',
        'test.json',
        'test2.json',
        'asd.txt',
        '111.json',
        'testMoved.json',
        'testBackup.json',
        str(Path.home()) + os.path.sep + 'lists.json',
        str(Path.home()) + os.path.sep + 'testMoved.json'
    ]
    removePaths = [
        'listOfLists',
        'lists',
    ]

    # remove json files incase they exist
    for file in removeFiles:
        if os.path.exists(file):
            os.remove(file)

    # remove directories incase they exist
    for path in removePaths:
        if os.path.exists(path):
            rmtree(path)


@pytest.fixture(autouse=True)
def setup_teardown():
    # setup
    remove()

    actions.createConfig()
    actions.createListList()

    yield  # runs tests

    # teardown
    remove()


def test_ListNameEdit():
    open('test.json', 'w').close()

    actions.ListNameEdit('test.json', 'test2.json')
    assert os.path.exists('test2.json')
    assert ('test2.json' in json.loads(open('lists.json').read())['lists'])
    assert not ('test.json' in json.loads(open('lists.json').read())['lists'])

    actions.ListNameEdit('test2.json', 'test2.json')
    assert os.path.exists('test2.json')
    assert ('test2.json' in json.loads(open('lists.json').read())['lists'])

    actions.ListNameEdit('test2.json', 'asd.txt')
    assert os.path.exists('asd.txt')
    assert ('asd.txt' in json.loads(open('lists.json').read())['lists'])
    assert not ('test2.json' in json.loads(open('lists.json').read())['lists'])

    actions.ListNameEdit('asd.txt', 'asd.txt')
    assert os.path.exists('asd.txt')
    assert ('asd.txt' in json.loads(open('lists.json').read())['lists'])

    actions.ListNameEdit('asd.txt', '111.json')
    assert os.path.exists('111.json')
    assert ('111.json' in json.loads(open('lists.json').read())['lists'])
    assert not ('asd.txt' in json.loads(open('lists.json').read())['lists'])

    actions.ListNameEdit('111.json', 'test.json')
    assert os.path.exists('test.json')
    assert ('test.json' in json.loads(open('lists.json').read())['lists'])
    assert not ('111.json' in json.loads(open('lists.json').read())['lists'])


def test_insert():

    actions.new('test.json')
    actions.insert('test.json', 'task', 1)
    assert json.loads(open('test.json').read())['tasks'][0] == ['task', 1]

    actions.insert('test.json', 'task2', 2)

    assert json.loads(open('test.json').read())['tasks'][1] == ['task2', 2]

    actions.insert('test.json', 'task1', 2)

    assert (json.loads(open('test.json').read())['tasks'][0] == ['task', 1])
    assert (json.loads(open('test.json').read())['tasks'][1] == ['task1', 2])
    assert (json.loads(open('test.json').read())['tasks'][2] == ['task2', 3])

    temp = json.loads(open('test.json').read())['tasks']

    actions.insert('test.json', 'task3', 5)
    # should not change the list
    assert json.loads(open('test.json').read())['tasks'] == temp


def test_changeListListPath():
    sep = os.path.sep

    # Create 2 lists
    actions.new('test.json')
    actions.new('test2.json')

    # add tasks to lists
    actions.addToList('test.json', 'task')
    actions.addToList('test2.json', 'task2')

    # change path of listList
    actions.changeListListPath('listList' + sep + 'listOfLists.json')
    # check if file exists
    assert(os.path.exists('listList' + sep + 'listOfLists.json'))
    # check if file was deleted
    assert(not os.path.exists('lists.json'))

    # check if the lists file's contents remain the same
    file = json.loads(open('listList' + sep + 'listOfLists.json').read())
    file = file['lists']
    assert('test.json' in file)
    assert('test2.json' in file)

    # move the lists file to home directory
    path = str(Path.home()) + os.path.sep + 'lists.json'
    # os.path.sep is the path separator, usually ' + sep + ' or \
    actions.changeListListPath(path)
    # check if file exists
    assert(os.path.exists(path))
    # check if file was deleted
    assert(not os.path.exists('listList' + sep + 'listOfLists.json'))

    # check if the lists file's contents remain the same
    assert('test.json' in json.loads(open(path).read())['lists'])
    assert('test2.json' in json.loads(open(path).read())['lists'])


def test_createConfig():
    # check if config file exists
    assert os.path.exists('config.ini')
    # read config file
    temp = open('config.ini').read()
    # create config file (should not change the file)
    actions.createConfig()
    # check if config file contents remain the same
    assert temp == open('config.ini').read()


def test_changeListPath():
    # create list
    actions.new('test.json')

    # add task to lists
    actions.addToList('test.json', 'task')

    # move and assert
    # save test.json as a variable
    temp = json.loads(open('test.json').read())
    # change path of list
    actions.changeListPath('test.json', 'testMoved.json')
    # assert
    assert os.path.exists('testMoved.json')\
        and not os.path.exists('test.json')\
        and json.loads(open('testMoved.json').read()) == temp

    # move to folder lists and assert
    # save list as variable
    temp = json.loads(open('testMoved.json').read())
    # change path of list
    path = 'lists' + os.path.sep + 'testMoved.json'
    actions.changeListPath('testMoved.json', path)
    # assert
    assert os.path.exists(path)\
        and not os.path.exists('testMoved.json')\
        and json.loads(open(path).read()) == temp

    # move to root directory and assert
    # save list as variable
    temp = json.loads(open('lists' + os.path.sep + 'testMoved.json').read())
    # change path of list
    prevPath = path
    path = str(Path.home()) + os.path.sep + 'testMoved.json'
    actions.changeListPath(prevPath, path)
    # assert
    assert os.path.exists(path)\
        and not os.path.exists(prevPath)\
        and json.loads(open(path).read()) == temp


def test_checkTaskExists():
    # create list
    actions.new('test.json')

    # add task to lists
    actions.addToList('test.json', 'task')

    # check if task exists
    assert actions.checkTaskExists('test.json', 'task')
    # check if task does not exist
    assert not actions.checkTaskExists('test.json', 'task2')
    # check if list does not exist
    assert not actions.checkTaskExists('test.json', 'task312')
    # check if list does not exist
    assert not actions.checkTaskExists('test.json', 'task22')


def test_addToList():
    # create list
    actions.new('test.json')

    # add task to test.json
    actions.addToList('test.json', 'task')

    # check if task exists
    assert actions.checkTaskExists('test.json', 'task')

    # add same task to test.json
    actions.addToList('test.json', 'task')

    # check if only one task exists
    file = getFile('test.json')
    assert len(file['tasks']) == 1
    assert file['tasks'][0] == ['task', 1]

    # add different task to test.json
    actions.addToList('test.json', 'task2')
    # check if two tasks exist
    file = getFile('test.json')
    assert len(file['tasks']) == 2
    assert file['tasks'][0] == ['task', 1]
    assert file['tasks'][1] == ['task2', 2]


def test_backup():
    actions.new('test.json')
    actions.addToList('test.json', 'task')
    actions.backup('test.json', 'testBackup.json')

    assert os.path.exists('testBackup.json')
    testBackup = json.loads(open('testBackup.json').read())
    test = json.loads(open('test.json').read())
    assert test == testBackup

    os.remove('test.json')

    assert not os.path.exists('test.json')
    assert os.path.exists('testBackup.json')


def test_restoreBackup():
    actions.new('test.json')
    actions.addToList('test.json', 'task')

    actions.backup('test.json', 'testBackup.json')

    testFile = json.loads(open('test.json').read())

    os.remove('test.json')

    actions.restoreBackup('testBackup.json', 'test.json')

    assert os.path.exists('test.json')

    restoredFile = json.loads(open('test.json').read())

    assert testFile == restoredFile



from termcolor import colored
import yato.JSONManip as JSONManip
import os
import json
import configparser
from shutil import copyfile


def main():
    return


def checkTaskExists(list, task):
    """"Checks if a task exists in a TODO list"""
    file = JSONManip.getFile(list)
    if task in file:
        return True
    return False


def warningDataLoss():
    print(colored('WARNING: Data will be lost.  Continue? (y/n)', 'red'))
    if input().lower() == 'y':
        return True
    else:
        return False


def help():
    print("yato - yet another TODO list\n\
    -h or --help:       show this help\n\
    -n or --new:        create a new TODO list\n\
    -a or --add:        add a task to a TODO list\n\
    -c or --complete:   complete a task\n\
    -r or --remove:     remove a task\n\
    -l or --list:       list all tasks\n\
    -ll or --list-all:  list all TODO lists\n\
    -d or --delete:     delete a TODO list\n\
    -da or --date:      add a date to a task\n\
    -e or --edit:       edit a TODO list's name\n\
    -i or --insert:     insert a task into a TODO list\n\
    -cll or --change-list-list: change the location of the list of lists\n\
    -cl or --change-list:     change the location of a TODO list\n\
    -b or --backup:     backup a TODO list\n\
    -rb or --restore:   restore a TODO list\n")


def new(fileLocation):
    json.dump({"todos": 0, "tasks": []}, open(fileLocation, 'w'))
    JSONManip.listListAdd(fileLocation.split('/')[-1])
    print('File created at: ' + fileLocation)


def addToList(list, add):
    if checkTaskExists(list, add):
        print(f'Task {add} already exists.')
        return
    JSONManip.changeTODOCount(list, True)
    file = JSONManip.getFile(list)
    file[add] = {'complete': False, 'index': file['todos']}
    file["tasks"].append((add, file['todos']))
    json.dump(file, open(list, 'w'))
    print(f'Task {add} added.')


def listTasks(fileLocation):
    file = JSONManip.getFile(fileLocation)
    for pos, task in enumerate(file['tasks']):
        task[1] = pos+1
        if file[task[0]]["complete"]:
            # print completed tasks green
            print(colored(str(task[1]) + '. ' + task[0], 'green'))
        else:
            # print uncompleted tasks red
            print(colored(str(task[1]) + '. ' + task[0], 'red'))


def completeTask(list, task):
    file = JSONManip.getFile(list)
    try:
        file[task]['complete'] = not file[task]['complete']
    except KeyError:
        print(f'Task {task} not found.')
    json.dump(file, open(list, 'w'))


def removeTask(list, task):
    try:
        # remove task from tasks list
        JSONManip.delTasksTask(list, task)
        # delete task from list
        JSONManip.delTask(list, task)
        # update todos
        JSONManip.changeTODOCount(list, False)
    except KeyError:
        print(f'Task {task} not found.')


def listAllLists():
    file = JSONManip.getFile('lists.json')
    for list in file['lists']:
        print(list)


def createListList():  # create lists.txt if it doesn't exist
    # open config.ini
    config = configparser.ConfigParser()
    config.read('config.ini')

    # create lists.txt if it doesn't exist
    if not os.path.exists(config['paths']['lists']):
        json.dump({"lists": []}, open('lists.json', 'w'))


def removeList(listPos):  # removes a TODO list
    if not warningDataLoss():
        return
    file = JSONManip.getFile('lists.json')
    try:
        file['lists'].remove(listPos)
        os.remove(listPos)
        json.dump(file, open('lists.json', 'w'))
        print(f'List {listPos} removed.')
    except ValueError:
        print(f'List {listPos} not found.')


def addDate(list, task, date):
    file = JSONManip.getFile(list)
    try:
        file[task]['date'] = [int(date[0]), int(date[1]), int(date[2])]
        json.dump(file, open(list, 'w'))
        date = '-'.join(date)
        print(f'Task {task} due at {date}.')
    except KeyError:
        print(f'Task {task} not found.')


def ListNameEdit(list, newName):
    os.rename(list, newName)
    file = JSONManip.getFile('lists.json')
    try:
        file['lists'].remove(list)
    except ValueError:
        pass
    file['lists'].append(newName)
    json.dump(file, open('lists.json', 'w'))


def insert(list, task, pos):
    file = JSONManip.getFile(list)
    # find highest index
    highest = 0
    for listTask in file['tasks']:
        if listTask[1] > highest:
            highest = listTask[1]
    if pos > highest+1:
        return print('Position too high.')
    # move up tasks in list that are greater or equal to pos
    firstPos = -1
    for fileTask in file['tasks']:
        if fileTask[1] >= pos:
            if firstPos == -1:
                firstPos = fileTask[1]
            fileTask[1] += 1
    # insert task into list
    if firstPos == -1:
        file['tasks'].insert(len(file['tasks'])+1, [task, pos])
    else:
        file['tasks'].insert(firstPos-1, [task, pos])
    file[task] = {'complete': False, 'index': pos}
    json.dump(file, open(list, 'w'))


def changeListListPath(newPath):
    # open config.ini
    config = configparser.ConfigParser()
    config.read('config.ini')

    # change path
    oldPath = config['paths']['lists']
    config['paths']['lists'] = newPath

    # write config.ini
    with open('config.ini', 'w') as configFile:
        config.write(configFile)

    # move file
    os.renames(oldPath, newPath)

    print('List list file location changed.')


def createConfig():
    if not os.path.exists('config.ini'):
        with open('config.ini', 'w') as config:
            config.write('[paths]\n')
            config.write("lists = lists.json \n")


def changeListPath(oldPath, newPath):
    # update lists.json
    file = JSONManip.getFile('lists.json')
    try:
        file['lists'].remove(oldPath)
    except ValueError:
        print(f'List {oldPath} not found, please check the path.')
        return

    file['lists'].append(newPath)
    json.dump(file, open('lists.json', 'w'))

    # move file
    os.renames(oldPath, newPath)

    print(f'List {oldPath} moved to {newPath}.')


def backup(original, backup):
    """Backup a list to a new location."""
    if os.path.exists(original):
        try:
            copyfile(original, backup)
        except FileNotFoundError:
            print(f'File {original} not found.')
        print(f'List {original} backed up to {backup}.')
    else:
        print(f'List {original} not found.')


def restoreBackup(backup, original):
    """Restore a list from a backup."""
    if os.path.exists(backup):
        try:
            copyfile(backup, original)
        except FileNotFoundError:
            print(f'File {backup} not found.')
        print(f'List {backup} restored to {original}.')
    else:
        print(f'List {backup} not found.')



import json
import sys


def getFile(filePos):
    try:
        return json.load(open(filePos))
    except FileNotFoundError:
        print('File not found.')
        sys.exit()
    except PermissionError:
        print('Permission denied.')
        sys.exit()


def changeTODOCount(fileName, add):
    file = getFile(fileName)
    if add:
        file['todos'] += 1
    else:
        file['todos'] -= 1
    json.dump(file, open(fileName, 'w'))
    return


def listListAdd(add):
    file = getFile('lists.json')
    file['lists'].append(add)
    json.dump(file, open('lists.json', 'w'))


def delTask(fileName, task):
    file = getFile(fileName)
    del file[task]
    json.dump(file, open(fileName, 'w'))


def delTasksTask(listLoc, task):
    file = getFile(listLoc)
    for lTask in file['tasks']:
        if lTask[0] == task:
            file['tasks'].remove(lTask)
            break
    json.dump(file, open(listLoc, 'w'))



import yato.actions as actions
import yato.get as get


def main():
    actions.createConfig()  # create config.ini if it doesn't exist
    actions.createListList()  # create lists.json if it doesn't exist

    # get command
    cmd = get.getCmd()

    if cmd == '-h' or cmd == '--help':  # help at help.txt
        actions.help()

    elif cmd == '-n' or cmd == '--new':  # New List
        actions.new(get.getFileLocation(2))

    elif cmd == '-a' or cmd == '--add':  # Add to list
        actions.addToList(get.getFileLocation(2), get.getTask(3))

    elif cmd == '-l' or cmd == '--list':  # List all tasks
        actions.listTasks(get.getFileLocation(2))

    elif cmd == '-c' or cmd == '--complete':  # Complete task
        actions.completeTask(get.getFileLocation(2), get.getTask(3))

    elif cmd == '-r' or cmd == '--remove':  # Remove task
        actions.removeTask(get.getFileLocation(2), get.getTask(3))

    elif cmd == '-ll' or cmd == '--list-all':  # lists all lists
        actions.listAllLists()

    elif cmd == '-d' or cmd == '--delete':  # Delete list
        actions.removeList(get.getFileLocation(2))

    elif cmd == '-da' or cmd == '--date':  # Add date to task
        actions.addDate(get.getFileLocation(2), get.getTask(3), get.getDate(4))

    elif cmd == '-e' or cmd == '--edit':  # Edit list name
        actions.ListNameEdit(get.getFileLocation(2), get.getNewName(3))

    elif cmd == '-i' or cmd == '--insert':  # Insert task
        actions.insert(get.getFileLocation(2), get.getTask(3), get.getuInt(4))

    elif cmd == '-cll' or cmd == '--change-list-list':
        actions.changeListListPath(get.getFileLocation(2))

    elif cmd == '-cl' or cmd == '--change-list':
        actions.changeListPath(get.getFileLocation(2), get.getFileLocation(3))

    elif cmd == '-b' or cmd == '--backup':
        actions.backup(get.getFileLocation(2), get.getFileLocation(3))

    elif cmd == '-rb' or cmd == '--restore-backup':
        actions.restoreBackup(get.getFileLocation(2), get.getFileLocation(3))


main()



from setuptools import setup

setup(
    name='yato',
    version='1.3.0',
    description='yato - Yet another Todo app, a cli program for TODO lists',
    url="https://github.com/TheArchons/yato",
    author='TheArchons',
    author_email='alexli9138@gmail.com',
    license='MIT',
    packages=['yato'],
    install_requires=['termcolor'],
    entry_points={'console_scripts': ['yato=yato.actions:main']},
    keywords=['yato', 'todo', 'list', 'tasks'],
)
